#include "system.h"
#include "UART_Library.h"
#include "G168_UART_Interface.h"
#include <string.h>
#include "Queue.h"

EX_INTERRUPT_HANDLER(ISR_UART_Rx);




extern Queue *cmdQueue;

/********************************************************************************/
/***** UART_Init()														*****/
/********************************************************************************/
void UART_Init(void)
{	int period, divisor, bitrate;
	char str[255];
	short temp;

#ifndef SKIP_AUTOBAUD_DETECTION
	period = UART_detectAutobaud();
#else
	period = 0x202c;
#endif

	// Apply formula divisor = period / (16 x 8 bits)
	divisor = period >> 7;
	UART_init(divisor);

	bitrate = SYSCLK/16/divisor;
	// \n\r is a CR, LF escape character combination
	UART_puts("********************************************************\n\r");
	UART_puts("Analog Devices Blackfin T1-E1 Interface\n\r");
	UART_puts("********************************************************\n\r");
    sprintf(str, "Detected UART host at %d baud\n\r", bitrate);
	UART_puts(str);
	
	
	// Set up thr UART interrupts
	*pILAT |= EVT_IVG10; // clear pending IVG10 interrupts
	*pSIC_IAR1 = P14_IVG(10);
	ssync();
	*pSIC_IMASK |= SIC_MASK(14);
	register_handler(ik_ivg10, ISR_UART_Rx); 

	temp = *pUART_RBR;
	temp = *pUART_LSR;
	temp = *pUART_IIR;
	// Setting the ETBEI bit automatically fires a TX interrupt request.
	*pUART_IER = ERBFI;
	ssync();
	
}


EX_INTERRUPT_HANDLER(ISR_UART_Rx)
{
	unsigned char inData;
	static char tempStr[YY_BUF_SIZE];
	static char *charPtr = tempStr;
		
	// read received value (also clears the interrupt)
	inData = *pUART_RBR;

	if (inData == 0x0d) { // 0x0d == ENTER
		*charPtr = 0x00;  // 0x00 == EOF
		//*charPtr = inData;
		enqueue(cmdQueue, tempStr);
		charPtr = tempStr;
		// confirm by sending the LF/CR combination to the UART
		UART_puts("\n\r");
		return;
	}
	else {
		*charPtr = inData;
		charPtr++;
		// confirm by sending the received value back via UART
		*pUART_THR = inData;
	}
	
}

/* Implementation of the Yacc buffer input function */
int pipe_yyinput(char buf[YY_BUF_SIZE], int max_size)
{
	int len;

	if (getQueueCount(cmdQueue) > 0) {
		dequeue(cmdQueue, buf);
		len = strlen(buf);
		//fprintf(stderr, "pipe_yyinput: %s (%d)", buf, len);
		//yy_flush_buffer();
		return len;
	}

	else {
		//fprintf(stderr, "pipe_yyinput: (0)");
		//yy_flush_buffer();
		return 0;
	}
}

/* Implementation of the Yacc buffer output function */
int pipe_yyoutput(char buf[], int len)
{
	UART_puts(buf);
	return 0;
}
